#ifndef LCD_KS0066_4BIT_IO_H
#define LCD_KS0066_4BIT_IO_H
/*************************************************************************
 Title	:   C include file for the KS0066U LCD library (lcd_ks0066_4bit_io.c)
 Author:    Original von Peter Fleury, anpassung Jürgen Oelkoch j.oe@gmx.de
 Software:  AVR-GCC 4.3.2
 Hardware:  any AVR device
***************************************************************************/
#if (__GNUC__ * 100 + __GNUC_MINOR__) < 303
#error "This library requires AVR-GCC 3.3 or later, update to newer AVR-GCC compiler !"
#endif

#include <inttypes.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

/*************************************************************************
 *  name  Definitions for Display Size 
 *  Change these definitions to adapt setting to your display
***************************************************************************/
#define LCD_LINES           2     /**< number of visible lines of the display */
#define LCD_DISP_LENGTH    20     /**< visibles characters per line of the display */
#define LCD_LINE_LENGTH  0x40     /**< internal line length of the display    */
#define LCD_START_LINE1  0x00     /**< DDRAM address of first char of line 1 */
#define LCD_START_LINE2  0x40     /**< DDRAM address of first char of line 2 */
//#define LCD_START_LINE3  0x14     /**< DDRAM address of first char of line 3 */
//#define LCD_START_LINE4  0x54     /**< DDRAM address of first char of line 4 */
#define LCD_WRAP_LINES      0     /**< 0: no wrap, 1: wrap at end of visibile line */

/*************************************************************************
 *  @name Definitions for 4-bit IO mode
 *  Change LCD_PORT if you want to use a different port for the LCD pins.
 *
 *  The four LCD data lines and the three control lines RS, RW, E can be on the 
 *  same port or on different ports. 
 *  Change LCD_RS_PORT, LCD_RW_PORT, LCD_E_PORT if you want the control lines on
 *  different ports. 
 *
 *  Normally the four data lines should be mapped to bit 0..3 on one port, but it
 *  is possible to connect these data lines in different order or even on different
 *  ports by adapting the LCD_DATAx_PORT and LCD_DATAx_PIN definitions.
 *  
***************************************************************************/
#define LCD_PORT         PORTD        /**< port for the LCD lines   */
#define LCD_DATA0_PORT   LCD_PORT     /**< port for 4bit data bit 0 */
#define LCD_DATA1_PORT   LCD_PORT     /**< port for 4bit data bit 1 */
#define LCD_DATA2_PORT   LCD_PORT     /**< port for 4bit data bit 2 */
#define LCD_DATA3_PORT   LCD_PORT     /**< port for 4bit data bit 3 */
#define LCD_DATA0_PIN    3            /**< pin for 4bit data bit 0  */
#define LCD_DATA1_PIN    4            /**< pin for 4bit data bit 1  */
#define LCD_DATA2_PIN    5            /**< pin for 4bit data bit 2  */
#define LCD_DATA3_PIN    6            /**< pin for 4bit data bit 3  */
#define LCD_RS_PORT      LCD_PORT     /**< port for RS line         */
#define LCD_RS_PIN       0            /**< pin  for RS line         */
#define LCD_RW_PORT      LCD_PORT     /**< port for RW line         */
#define LCD_RW_PIN       1            /**< pin  for RW line         */
#define LCD_E_PORT       LCD_PORT     /**< port for Enable line     */
#define LCD_E_PIN        2            /**< pin  for Enable line     */

/*************************************************************************
 *  name Definitions for LCD command instructions
 *  The constants define the various LCD controller instructions which can be passed to the 
 *  function lcd_command(), see KS0066U data sheet for a complete description.
***************************************************************************/

/* instruction register bit positions, see HD44780U data sheet */
#define LCD_CLR               0      /* DB0: clear display                  */
#define LCD_HOME              1      /* DB1: return to home position        */
#define LCD_ENTRY_MODE        2      /* DB2: set entry mode                 */
#define LCD_ENTRY_INC         1      /*   DB1: 1=increment, 0=decrement     */
#define LCD_ENTRY_SHIFT       0      /*   DB2: 1=display shift on           */
#define LCD_ON                3      /* DB3: turn lcd/cursor on             */
#define LCD_ON_DISPLAY        2      /*   DB2: turn display on              */
#define LCD_ON_CURSOR         1      /*   DB1: turn cursor on               */
#define LCD_ON_BLINK          0      /*     DB0: blinking cursor ?          */
#define LCD_MOVE              4      /* DB4: move cursor/display            */
#define LCD_MOVE_DISP         3      /*   DB3: move display (0-> cursor) ?  */
#define LCD_MOVE_RIGHT        2      /*   DB2: move right (0-> left) ?      */
#define LCD_FUNCTION          5      /* DB5: function set                   */
#define LCD_FUNCTION_8BIT     4      /*   DB4: set 8BIT mode (0->4BIT mode) */
#define LCD_FUNCTION_2LINES   3      /*   DB3: two lines (0->one line)      */
#define LCD_FUNCTION_10DOTS   2      /*   DB2: 5x10 font (0->5x7 font)      */
#define LCD_CGRAM             6      /* DB6: set CG RAM address             */
#define LCD_DDRAM             7      /* DB7: set DD RAM address             */
#define LCD_BUSY              7      /* DB7: LCD is busy                    */

/* set entry mode: display shift on/off, dec/inc cursor move direction */
#define LCD_ENTRY_DEC            0x04   /* display shift off, dec cursor move dir */
#define LCD_ENTRY_DEC_SHIFT      0x05   /* display shift on,  dec cursor move dir */
#define LCD_ENTRY_INC_           0x06   /* display shift off, inc cursor move dir */
#define LCD_ENTRY_INC_SHIFT      0x07   /* display shift on,  inc cursor move dir */

/* display on/off, cursor on/off, blinking char at cursor position */
#define LCD_DISP_OFF             0x08   /* display off                            */
#define LCD_DISP_ON              0x0C   /* display on, cursor off                 */
#define LCD_DISP_ON_BLINK        0x0D   /* display on, cursor off, blink char     */
#define LCD_DISP_ON_CURSOR       0x0E   /* display on, cursor on                  */
#define LCD_DISP_ON_CURSOR_BLINK 0x0F   /* display on, cursor on, blink char      */

/* move cursor/shift display */
#define LCD_MOVE_CURSOR_LEFT     0x10   /* move cursor left  (decrement)          */
#define LCD_MOVE_CURSOR_RIGHT    0x14   /* move cursor right (increment)          */
#define LCD_MOVE_DISP_LEFT       0x18   /* shift display left                     */
#define LCD_MOVE_DISP_RIGHT      0x1C   /* shift display right                    */

#define LCD_MODE_DEFAULT     ((1<<LCD_ENTRY_MODE) | (1<<LCD_ENTRY_INC) )

/*************************************************************************
Initialize display and select type of cursor
 param    dispAttr \b LCD_DISP_OFF display off\n
                    \b LCD_DISP_ON display on, cursor off\n
                    \b LCD_DISP_ON_CURSOR display on, cursor on\n
                    \b LCD_DISP_ON_CURSOR_BLINK display on, cursor on flashing
 return  none
***************************************************************************/
extern void lcd_init(uint8_t dispAttr);


/*************************************************************************
 Clear display and set cursor to home position
 param    void                                        
 return   none
***************************************************************************/
extern void lcd_clrscr(void);


/*************************************************************************
 Set cursor to home position
 param    void                                        
 return   none
***************************************************************************/
extern void lcd_home(void);


/*************************************************************************
  Set cursor to specified position
 
 param    x horizontal position\n (0: left most position)
 param    y vertical position\n   (0: first line)
 return   none
***************************************************************************/
extern void lcd_gotoxy(uint8_t x, uint8_t y);


/*************************************************************************
 Display character at current cursor position
 param    c character to be displayed                                       
 return   none
***************************************************************************/
extern void lcd_putc(char c);


/*************************************************************************
 Display string without auto linefeed
 param    s string to be displayed                                        
 return   none
***************************************************************************/
extern void lcd_puts(const char *s);


/*************************************************************************
 Display string from program memory without auto linefeed
 param    s string from program memory be be displayed                                        
 return   none
 see      lcd_puts_P
***************************************************************************/
extern void lcd_puts_p(const char *progmem_s);


/*************************************************************************
 Send LCD controller instruction command
 param    cmd instruction to send to LCD controller, see HD44780 data sheet
 return   none
***************************************************************************/
extern void lcd_command(uint8_t cmd);


/*************************************************************************
 Send data byte to LCD controller 
 
 Similar to lcd_putc(), but without interpreting LF
 param    data byte to send to LCD controller, see HD44780 data sheet
 return   none
***************************************************************************/
extern void lcd_data(uint8_t data);

/*************************************************************************
schreibt eine Leerzeile mit LCD_DISP_LENGTH Leerzeichen
zuvor mus Cursor mit lcd_gotoxy an den Anfang der gewünschten Zeile
positioniert werden 
Input:    none
Returns:  none
*************************************************************************/
extern void lcd_leerzeile(void);

/*************************************************************************
 macros for automatically storing string constant in program memory
***************************************************************************/
#define lcd_puts_P(__s)         lcd_puts_p(PSTR(__s))

#endif
