/*************************************************************************
 Title	:   C include file for led fading library (led.c)
 Author:    Jürgen Oelkoch j.oe@gmx.de
 Software:  AVR-GCC 4.3.2
 Hardware:  any AVR device
***************************************************************************/

#include <inttypes.h>
#include <avr/pgmspace.h>

//Anzahl Programme, die der Fader ausführt
#define ANZ_PROGRAMME 6
//Anzahl Kanäle
#define KANAELE 12
//Anzahl Verlaufsstufen
#define ANZ_VERLAUF 6
//Anzahl dimmbarer Stufen
#define PWM_STUFEN 256

// ABGLEICH_ROT		->Variablen zum Ausgleich der Leuchtstärke der einzelnen
// ABGLEICH_GRUEN	  Farben; wichtig, um definierte Farbtöne modellieren zu können 
// ABGLEICH_BLAU	  Werte dienen zur Skalierung des 256-stufigen-Dimmwertes, der aus dem
// 			  16-bit-Array pwmtable_16[PWM_STUFEN] entnommenen wird, um den
// 			  16-bit-Wert für die PCM-gesteuerte Helligkeit auszugeben.
// 			  Skalierung über Dreisatz pwmtable[x]*ABGLEICH_xxx/255
// 			->Wert < 255, entsprechend dunklere Grundeinstellung der LED  
#define ABGLEICH_ROT   255
#define ABGLEICH_GRUEN 255
#define ABGLEICH_BLAU  255

//-------------------------------------------------------------------------------------------------------
//Typdefinition für die
//Definition eines Variablensets für einen Fading-Verlauf
//
// name			->Name des Fading-sets
// 
// kanaele[][]		->Der jeweilige Verlauf des Kanals wird als Zeichenfolhe von 0 und 1 dargestellt
// 			  Dabei gilt:
//			  0 -> 1 Einblendung in der Anzahl dimmbarer Stufen
//  			  1 -> 0 Ausblendung    "    "       "         "
//  			  1 -> 1 bleibt an
//  			  0 -> 0 bleibt aus
//-------------------------------------------------------------------------------------------------------
typedef struct {
    char name[21];
    uint8_t kanaele[KANAELE][ANZ_VERLAUF];
    uint8_t lin_log;
} led_set;

//-------------------------------------------------------------------------------------------------------
//Typdefinition für die
//Definition eines Variablensets für die Geschwindigkeit jedes Fading-Verlaufes
//
// name			->Name der Tempostufe (angezeigt in der 2. Displayzeile
// 
// schritt_delay	->ein Fadingdurchlauf besteht aus 6 Abschnitten mit je 256 Schritten
// 			  also gilt für einen durchlauf: 1+254+1+254+1+254+1+254+1+254+1+254 = 1530 Schritte
//			  Diese Anzahl mal schritt_delay ergibt die Zeit für einen Fading-Druchlauf
//  			  117 ms -> 3  min
//  			   78 ms -> 2  min
//  			   59 ms -> 90 sek
//  			   39 ms -> 60 sek
//  			   26 ms -> 40 sek
//  			   20 ms -> 30 sek
//  			   13 ms -> 20 sek
//  			   10 ms -> 15 sek
//  			    7 ms -> 10 sek
//  			    3 ms ->  4 sek
//-------------------------------------------------------------------------------------------------------
typedef struct {
    char name [21];
    uint8_t anz_ms;
} tempo_stufe;

//-------------------------------------------------------------------------------------------------------
//Typdefinition für die
//Definition eines Variablensets für konstante Farben ohne Fading
//
// name			->Bezeichnung der Farbe (angezeigt in der 2. Displayzeile
// 
// rot			->Dimmwerte für die Einzelfarben r, g und b
// gruen
// blau
// 
// abgleich_rot		->Abgleichwerte für die Einzelfarben r, g und b damit bei Mischfarben
// abgleich_gruen	  der Helligkeitsunterschied der einzelnen LED-Farben ausgeglichen werden kann.
// abgleich_blau	  Werte entweder 255 (volle Helligkeit der Einzelfarbe) oder jeweiliger Wert
// 			  der in Abgleichkonstanten in #define  
//-------------------------------------------------------------------------------------------------------
typedef struct {
    char name [21];
    uint8_t rot;
    uint8_t gruen;
    uint8_t blau;
    uint8_t abgleich_rot;
    uint8_t abgleich_gruen;
    uint8_t abgleich_blau;
} farb_set;

//-------------------------------------------------------------------------------------------------------
//Funktionsprototypen
//-------------------------------------------------------------------------------------------------------
void fading(led_set *set);
void konstante_farbe(void);
void init_tempo_und_farbe(void);
void alles_aus(void);


